/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * EE to test events, looks suspiciously like chantest...
 */

#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_flow.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_event.h>
#include <nodeos/an_mempool.h>

static void ee_init(void *arg);
an_flowinit_func_t InitEE = { ee_init };

static void setup_events(void);
static void ee_event1(void *event, void *arg);
static void ee_event100(void *event, void *arg);
static unsigned long eventres;

int
ee_main(int argc, const char **argv)
{
	printf("%p/%p: Channel test EE started\n",
	       an_flow_current(), an_thread_current());

	/*
	 * Setup events
	 */
	setup_events();

	/*
	 * No longer need this thread
	 */
	return 0;
}

static void
setup_events(void)
{
	an_error_t err;

	eventres = an_event_granularity();
	printf("Event granularity: %ld us (%ld per sec max)\n",
	       eventres, 1000000 / eventres);

	/* max resolution */
	err = an_event_schedule(ee_event1, (void *)1, eventres);
	assert(err == 0);

	/* every second */
	err = an_event_schedule(ee_event100, (void *)100, 1000000);
	assert(err == 0);
}

static int events;

static void
ee_event1(void *event, void *arg)
{
	an_error_t err;

	assert((int)arg == 1);

	events++;
	err = an_event_schedule(ee_event1, (void *)1, eventres);
	assert(err == 0);
}

static void
ee_event100(void *event, void *arg)
{
	static int count;
	an_timespec_t tspec;
	int err;
	
	assert((int)arg == 100);

	an_gettime(&tspec);
	printf("%p/%p: got %d events at %u seconds\n",
	       an_flow_current(), an_thread_current(), events,
	       (unsigned int)tspec.tv_sec);

	if (++count == 20) {
		printf("killing our flow %p\n", an_flow_current());
		an_flow_destroy(an_flow_current());
	} else {
		err = an_event_schedule(ee_event100, (void *)100, 1000000);
		assert(err == 0);
	}
}

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	const char *argv[] = { "ChantestEE" }; 

	status = ee_main(argc, argv);
}

