/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
/*
 * rootflow.c --- test parts of the root flow.
 */

#include <stdio.h>
#include <stdlib.h>

#include <nodeos/an_flow.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_moab_setup.h>

static void rootflow_threadfunc(void *arg);

static an_cred_t root_cred;
static an_cred_t initee_cred;
static an_mempool_t root_mempool;
static an_flow_t root_flow;

#define ROOTFLOW_THREAD_STACKSIZE  (20 * 1024) /* OSKit pthread minimum */

typedef struct { char stack[ROOTFLOW_THREAD_STACKSIZE];} stackhack_t;

static an_thread_t prealloced_threads[5];
static stackhack_t prealloced_stacks[5];

/*
 * Allocates N threads in root flow (N defaults to 1).
 */
int
main(int argc, char **argv)
{
	void* threads[5];
	void* stacks[5];
	int threadct = 1;
	int i;

	if (argc > 1)
		threadct = atoi(argv[1]);

	assert(threadct <= 5);
	assert(threadct > 0);

	for (i = 0; i < threadct; i++)
	{
		threads[i] = &prealloced_threads[i];
		memset(&prealloced_threads[i], 0, ROOTFLOW_THREAD_STACKSIZE);
		stacks[i] = &prealloced_stacks[i];
	}

	/*
	 * This code all runs without a flow or NodeOS thread
	 * context.  There are few things that can be done
	 * in this context.
	 */
	an_moab_setup();
	
	printf("Utah NodeOS (version 0.%04d)\n", an_moab_version());

	an_moab_start(&root_flow, &root_mempool,
		      threadct, ROOTFLOW_THREAD_STACKSIZE,
		      threads, stacks,
		      &root_cred,
		      &initee_cred,
		      rootflow_threadfunc, /*load_ee arg: */NULL);
	
	printf("Root flow died, exiting.\n");

	return 0;
}

static void
rootflow_threadfunc(void *arg)
{
	printf("First (and only) thread in root flow started.\n");

	an_moab_shutdown();
}
