/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Condition variable API.
 */
#ifndef _NODEOS_COND_H_
#define _NODEOS_COND_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_Cond an_cond_t
#define an_condCreate an_cond_create
#define an_condWait an_cond_wait
#define an_condTimedWait an_cond_timedwait
#define an_condSignal an_cond_signal
#define an_condDestroy an_cond_destroy
#endif

#include <nodeos/an_error.h>
#include <nodeos/an_time.h>
#include <nodeos/an_mutex.h>
#include <nodeos/impl/an_cond.h>

/*
 * Initialize an existing hunk to be a condition variable object.
 */
an_cond_t *an_cond_create(void *mem);

/*
 * Wait for a condition to be signalled.
 * Atomically releases the specified mutex and blocks (interruptibly)
 * until signalled.  Spurious wakeups are possible and, in fact, will
 * occur if the thread is an_thread_intr'ed while in a wait.
 */
void an_cond_wait(an_cond_t *cond, an_mutex_t *mutex);

/*
 * Wait until the given absolute time as expired or until the cond
 * is signaled.  No indication of what happened is returned. (XXX?)
 * Atomically releases the specified mutex and blocks (interruptibly)
 * until woken.  Spurious wakeups are possible and, in fact, will
 * occur if the thread is an_thread_intr'ed while in a wait.
 */
void an_cond_timedwait(an_cond_t *cond, an_mutex_t *mutex,
		       an_timespec_t *abstime);

/*
 * Wakeup a single thread blocked on a condition variable.
 */
void an_cond_signal(an_cond_t *cond);

/*
 * Wakeup all threads blocked on a condition variable.
 */
void an_cond_broadcast(an_cond_t *cond);

/*
 * Destroy a condition variable.
 * Returns zero if successful, an error if it could not be destroyed.
 */
an_error_t an_cond_destroy(an_cond_t *cond);

#endif /* _NODEOS_COND_H_ */
