/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Demuxkey API
 */
#ifndef _NODEOS_DEMUXKEY_H_
#define _NODEOS_DEMUXKEY_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_Demuxkey an_demuxkey_t
#define an_demuxkeyCreate an_demuxkey_create
#define an_demuxkeyDestroy an_demuxkey_destroy
#define an_demuxkeyMatch an_demuxkey_match
#endif

#include <nodeos/an_impl.h>
#include <nodeos/an_flow.h>
#include <nodeos/impl/an_demuxkey.h>

/*
 * Create an empty demux key (packet filter) object.
 * Returns a pointer to the new object or zero on error.
 */
an_demuxkey_t *an_demuxkey_create(void *mem, an_flow_t *flow);

/*
 * Destroy a demux key.
 * Frees any resources allocated by an_demuxkey_match operations.
 * XXX make sure demux key is not "in use".
 */
an_error_t an_demuxkey_destroy(an_demuxkey_t *key);

/*
 * Clone a demux key.  New demux key is created in the provided memory.
 * Returns a pointer to the new object or zero on error.
 */
an_demuxkey_t *an_demuxkey_clone(void *mem, an_demuxkey_t *orig);

/*
 * Adds an <offset, length, matchdata> triple to the indicated demux.
 * The match data buffer is currently copied and hence may be overwritten
 * upon return from this call.  The mask param is currently ignored.
 */
an_error_t an_demuxkey_add(an_demuxkey_t *key, int off, int len,
			   void *data, void *mask);

/*
 * Adds an IPv4 header match segment to the indicated demux key.
 * This key matches a particular IPv4 header (with the required
 * source, dest, and protocol fields).
 */
an_error_t an_demuxkey_addipv4(an_demuxkey_t *key, int off, 
			       int source, int dest, int protocol);

/* XXX finish proto */
an_error_t an_demuxkey_addudp(void);

/* XXX finish proto */
an_error_t an_demuxkey_addtcp(void);

/* XXX finish proto */
an_error_t an_demuxkey_addanep(void);

/*
 * Test a data buffer against the given key.
 * Returns -1 if there was an error, otherwise the index
 * of the first byte that was not part of the match.  (Returns
 * 0 if there was no match.)
 * 'data' is the chunk of data to compare against.
 * 'datalength' is the length of data.
 *
 * XXX doesn't work with "null" keys.  Should banish null keys?
 */
int an_demuxkey_matches(an_demuxkey_t *key, void *data, int datalength);

#endif /* _NODEOS_DEMUXKEY_H_ */
