/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Mutex API.
 */
#ifndef _NODEOS_MUTEX_H_
#define _NODEOS_MUTEX_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_Mutex an_mutex_t
#define an_mutexCreate an_mutex_create
#define an_mutexLock an_mutex_lock
#define an_mutexTrylock an_mutex_trylock
#define an_mutexUnlock an_mutex_unlock
#define an_mutexDestroy an_mutex_destroy
#endif

#include <nodeos/an_error.h>
#include <nodeos/impl/an_mutex.h>

/*
 * Initialize an existing object to be a valid mutex object.
 */
an_mutex_t *an_mutex_create(void *mem);

/*
 * Lock a mutex object.
 * Blocks (uninterruptibly) if the mutex is held.
 */
void an_mutex_lock(an_mutex_t *mutex);

/*
 * Attempt to lock a mutex.
 * Returns 0 if the lock succeeds, non-zero if the lock was already held.
 */
an_error_t an_mutex_trylock(an_mutex_t *mutex);

/*
 * Unlocks a mutex.
 */
void an_mutex_unlock(an_mutex_t *mutex);

/*
 * Destroys a mutex.
 * Fails and returns non-zero if the mutex is currently locked.
 */
an_error_t an_mutex_destroy(an_mutex_t *mutex);


#endif /* _NODEOS_MUTEX_H_ */
