/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Resource descriptors
 */
#ifndef _NODEOS_RESOURCE_H_
#define _NODEOS_RESOURCE_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_ThreadPoolSpec an_cpuspec_t
#define an_CpuSpec an_cpuspec_t
#define an_MemSpec an_memspec_t
#endif

#include <nodeos/an_impl.h>
#include <nodeos/an_time.h>
#include <nodeos/an_mempool.h>

/* XXX cannot include an_thread.h because of circular dependencies */
struct an_thread;

/*
 * Description of flow CPU resources.
 * Passed to the NodeOS when creating a new flow.  Includes a number of 
 * threads to use and information on how to schedule them.  Threads may
 * be described in one of two ways:
 *
 *	1. Explicitly: A list of explicit thread objects is passed in.
 *	   This method is used when the EE wants to preallocate thread
 *	   objects and their stacks.  In this model, the EE controls
 *	   placement of thread-related memory.
 *
 *	2. Implicitly: A limit on the maximum number of threads to allocate
 *	   and a maximum stack size to use for those threads is passed.
 *	   Useful if the EE doesn't care about the placement of thread
 *	   objects or maybe doesn't even need an explicit notion of a
 *	   thread.
 */
enum an_threadtype_t { AN_TH_IMPLICIT, AN_TH_EXPLICIT };
typedef struct an_cpuspec {
	/* description of threads */
	enum an_threadtype_t ttype;		/* type of threads */
	union {
		/* AN_TH_IMPLICIT */
		struct {
			int maxthreads;		/* max # of threads */
			size_t stacksize;	/* max stack size of threads */
		} im;
		/* AN_TH_EXPLICIT */
		struct {
			int nthreads;		    /* # of threads in list */
			struct an_thread **threads; /* list of thread objects */
		} ex;
	}		threads;
	/* scheduler params */
	int		rate;		/* max execution rate */
	an_timespec_t	slice;		/* max time slice */
	an_sched_t	*sched;		/* scheduler object */
} an_cpuspec_t;

/*
 * Description of flow (physical) memory resources.
 * Passed to the NodeOS when creating a new flow.  Includes a memory pool
 * with which the flow should be associated and the amount of memory
 * contributed to the memory pool for use by this flow.
 */
typedef struct an_memspec {
	int		npages;		/* physical pages to give to pool */
	an_mempool_t	*pool;		/* memory pool object */
} an_memspec_t;

/*
 * Description of outchan network bandwidth resources.
 * Passed to the NodeOS when creating a new outchan.  As of yet unspecified.
 */
typedef struct an_netspec {
	void		*qos;
} an_netspec_t;

#endif /* _NODEOS_RESOURCE_H_ */
