/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Implementation bits of NodeOS mutex API.
 */
#ifndef _NODEOS_IMPL_AN_MUTEX_H_
#define _NODEOS_IMPL_AN_MUTEX_H_

#include <nodeos/impl/decls.h>
#include <nodeos/an_error.h>
#include <nodeos/impl/ani_nodeos.h>
#include <nodeos/impl/ani_mutex.h>

/*
 * an_mutex interfaces.  
 *
 * an_mutexes are a simple wrapper around the ani_mutex layer.
 * This layer add the NODEOS entry/exit and INTER/EXTERN to the 
 * ani_mutex.
 *
 * The Utah NodeOS inlines the lock(), unlock() and trylock()
 * operations.  The init() and destroy() operations are not inlined.
 */

typedef struct an_mutex {
	ani_mutex_t mutex;	/* must be first for INTERN/EXTERN */
} an_mutex_t;

/* an_mutex inline functions */

NODEOS_FUNC_INLINE ani_mutex_t *
NODEOS_INTERN_MUTEX(an_mutex_t *m)
{
	assert(m != 0);
	return &m->mutex;
}

NODEOS_FUNC_INLINE an_mutex_t *
NODEOS_EXTERN_MUTEX(ani_mutex_t *im)
{
	assert(im != 0);
	return (an_mutex_t *)im;
}

NODEOS_FUNC_INLINE void 
an_mutex_lock(an_mutex_t *m)
{
	NODEOS_ENTER;
	ani_mutex_lock(&m->mutex);
	NODEOS_LEAVE;
}


NODEOS_FUNC_INLINE int
an_mutex_trylock(an_mutex_t *m)
{
	int rv;

	NODEOS_ENTER;
	rv = ani_mutex_trylock(&m->mutex);
	NODEOS_LEAVE;
	return rv;
}

NODEOS_FUNC_INLINE void 
an_mutex_unlock(an_mutex_t *m)
{
	NODEOS_ENTER;
	ani_mutex_unlock(&m->mutex);
	NODEOS_LEAVE;
}

#endif /* _NODEOS_IMPL_AN_MUTEX_H_ */


