/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Implementation bits of NodeOS pbuf API.
 */
#ifndef _NODEOS_IMPL_AN_PBUF_H_
#define _NODEOS_IMPL_AN_PBUF_H_

#include <nodeos/impl/decls.h>
#include <nodeos/impl/ani_nodeos.h>
#include <nodeos/impl/ani_pbuf.h>

/*
 * an_pbuf interfaces.  
 *
 * an_pbufs are a simple wrapper around the ani_pbuf layer.
 * This layer add the NODEOS entry/exit and INTERN/EXTERN to the
 * ani_pbuf.
 *
 * Currently we only inline the functions to return data/size.
 * The various add/subtract space calls would be potential candidates.
 */

typedef struct an_pbuf {
	ani_pbuf_t pbuf;
} an_pbuf_t;

/* an_pbuf inline functions */

NODEOS_FUNC_INLINE ani_pbuf_t *
NODEOS_INTERN_PBUF(an_pbuf_t *xpbuf)
{
	assert(xpbuf != 0);
	return &xpbuf->pbuf;
}

NODEOS_FUNC_INLINE an_pbuf_t *
NODEOS_EXTERN_PBUF(ani_pbuf_t *ipbuf)
{
	assert(ipbuf != 0);
	return (an_pbuf_t *)ipbuf;
}

NODEOS_FUNC_INLINE void *
an_pbuf_data(an_pbuf_t *pbuf)
{
	void *data;

	LW_NODEOS_ENTER;
	data = ani_pbuf_data(&pbuf->pbuf);
	LW_NODEOS_LEAVE;

	return data;
}

NODEOS_FUNC_INLINE int
an_pbuf_size(an_pbuf_t *pbuf)
{
	int size;

	LW_NODEOS_ENTER;
	size = (int)ani_pbuf_size(&pbuf->pbuf);
	LW_NODEOS_LEAVE;

	return size;
}

NODEOS_FUNC_INLINE int
an_pbuf_offset(an_pbuf_t *pbuf)
{
	int offset;

	LW_NODEOS_ENTER;
	offset = (int)ani_pbuf_offset(&pbuf->pbuf);
	LW_NODEOS_LEAVE;

	return offset;
}

NODEOS_FUNC_INLINE int
an_pbuf_bufsize(an_pbuf_t *pbuf)
{
	int size;

	LW_NODEOS_ENTER;
	size = (int)ani_pbuf_bufsize(&pbuf->pbuf);
	LW_NODEOS_LEAVE;

	return size;
}

#endif /* _NODEOS_IMPL_AN_PBUF_H_ */
