/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Credentials: as of yet ill-defined
 *
 * XXX dealing with resources here may be way off base.
 */
#ifndef _CRED_IMPL_H_
#define _CRED_IMPL_H_

#include <nodeos/impl/ani_error.h>
#include <nodeos/impl/ani_resource.h>

struct ani_cred {
	void		*stuff;		/* XXX authentication stuff */
	resource_t	current;	/* resources allotted to this "user" */
	resource_t 	limits;		/* resource limits for this "user" */
};
typedef struct ani_cred ani_cred_t;

/* credential_check operations */
#define CCHECK_FLOW_CREATE	1

/* credential_check returns */
#define CCHECK_OK	0
#define CCHECK_FAIL	1

NODEOS_FUNC_INLINE void assert_ani_cred_valid(ani_cred_t* cred);
void ani_cred_setup(ani_cred_t *root_cred, ani_cred_t *user_cred);
ani_error_t ani_cred_init(ani_cred_t* cred, void *stuff, resource_t *limits);
void ani_cred_deinit(ani_cred_t *cred);
int ani_cred_check(ani_cred_t *cred, int op, ...);
ani_error_t ani_cred_resource_alloc(ani_cred_t *cred, resource_t *res); 
void ani_cred_resource_free(ani_cred_t *cred, resource_t *res);
struct ani_cred *ani_cred_current(void);

NODEOS_FUNC_INLINE void
assert_ani_cred_valid(ani_cred_t *cred) 
{
	assert(cred != 0);

#ifdef DEBUG

	/* Current usage can be a bit below maximum */
	/* 
	 * XXX of course, most of these limits are inherently bogus.
	 * There's no reason you cannot have more than 100 threads.
	 * But, that shouldn't happen for a long, long time...
	 */
	assert(cred->current.threads   >= 0);
	assert(cred->current.threads   < 1001);
	assert(cred->current.cyclerate >= 0);
	assert(cred->current.cyclerate < 10001);
	assert(cred->current.timeslice >= 0);
	assert(cred->current.timeslice < 10001);
	assert((cred->current.stacksize == 0)
	       || (cred->current.stacksize > 20 * 1024)); // oskit minimum
	assert(cred->current.stacksize < 128 * 1024);
	assert(cred->current.ppages    >= 0);
	assert(cred->current.ppages    < 100 * 1024);

	/* Maximum limits are taken from an_cred_setup() */
	assert(cred->limits.threads   > 0);
	assert(cred->limits.threads   < 255);
	assert(cred->limits.cyclerate > 0);
	assert(cred->limits.cyclerate < 10001);
	assert(cred->limits.timeslice > 0);
	assert(cred->limits.timeslice < 10001);
	assert(cred->limits.stacksize > 20 * 1024); // oskit minimum
	assert(cred->limits.stacksize < 128 * 1024);
	assert(cred->limits.ppages    > 0); // oskit minimum
	assert(cred->limits.ppages    < 100 * 1024);
#endif /* DEBUG */
}

#endif /* _CRED_IMPL_H_ */
