/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS mutex entrypoints
 */

#include <nodeos/an_mutex.h>
#include <nodeos/an_error.h>
#include <nodeos/impl/ani_mutex.h>
#include <nodeos/impl/ani_flow.h>
#include <nodeos/impl/ani_mempool.h>

/*
 * Initalize an existing piece of memory to be a mutex object.
 */
an_mutex_t *
an_mutex_create(void *mem)
{
	ani_mutex_t *imux;
	ani_mempool_t *cpool;
	int didalloc = 0;
	ani_error_t err;

	NODEOS_ENTER;

	/*
	 * Allocate memory for the object if needed.
	 */
	cpool = ani_flow_mempool(ani_flow_current());
	if (mem == 0) {
		mem = ani_mempool_alloc(cpool, sizeof(an_mutex_t));
		didalloc = 1;
	}

	imux = NODEOS_INTERN_MUTEX(mem);
	if (imux == 0)
		err = ani_error_set(AN_ERR_NOMEMORY,
				    "invalid mutex argument");
	else
		err = ani_mutex_init(imux);

	if (ani_error_occured(err) && didalloc)
		ani_mempool_free(cpool, mem, sizeof(an_mutex_t));

	NODEOS_LEAVE;
	return ani_error_occured(err) ? 0 : NODEOS_EXTERN_MUTEX(imux);
}

/* an_mutex_lock is in the public header file <nodeos/impl/synch.h> */
/* an_mutex_trylock is in the public header file <nodeos/impl/synch.h> */
/* an_mutex_unlock is in the public header file <nodeos/impl/synch.h> */

an_error_t
an_mutex_destroy(an_mutex_t *mutex)
{
	ani_error_t err;
	ani_mutex_t *imux;

	NODEOS_ENTER;
	imux = NODEOS_INTERN_MUTEX(mutex);
	if (imux == 0)
		err = ani_error_set(AN_ERR_INVALIDARG,
				    "invalid mutex argument");
	else
		err = ani_mutex_deinit(imux);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}
