/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Credential manipulation.
 *
 * XXX total hack
 */
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <nodeos/impl/an_error.h>
#include <nodeos/impl/ani_resource.h>
#include <nodeos/impl/ani_cred.h>
#include <nodeos/impl/ani_flow.h>

void
ani_cred_setup(ani_cred_t *root_cred, ani_cred_t *user_cred)
{
	resource_t res;
	ani_error_t err;

	res.threads = 250;
	res.cyclerate = 10000;
	res.timeslice = 10000;
	res.stacksize = 64 * 1024;
	res.ppages = 100000;
	err = ani_cred_init(root_cred, NULL, &res);
	assert(!ani_error_occured(err));

	res.threads = 128;
	res.cyclerate = 1000;
	res.timeslice = 10000;
	res.stacksize = 64 * 1024;
	res.ppages = 10000;
	err = ani_cred_init(user_cred, (void*)1, &res);
	assert(!ani_error_occured(err));

	assert_ani_cred_valid(root_cred);
	assert_ani_cred_valid(user_cred);
}

ani_error_t
ani_cred_init(ani_cred_t* cred, void *stuff, resource_t *limits)
{
	if (cred == 0 || limits == 0)
		return ani_error_set(EINVAL,
				     "invalid argument to ani_cred_init");

	memset(cred, 0, sizeof *cred); /* XXX ? */
	cred->limits = *limits;

	assert_ani_cred_valid(cred);
	return ani_error_clear();
}

void
ani_cred_deinit(ani_cred_t *cred)
{
	assert_ani_cred_valid(cred);
}

ani_cred_t *
ani_cred_current(void)
{
	ani_cred_t *cred;
	cred = ani_flow_current()->cred;
	assert_ani_cred_valid(cred);
	return cred;
}

int
ani_cred_check(ani_cred_t *cred, int op, ...)
{
	assert_ani_cred_valid(cred);
	return CCHECK_OK;
}

/* return 0 if adding amount from res doesn't exceed the cred limit */
#define credres_check(c,r,f) \
	((r)->f + (c)->current.f > (c)->limits.f)

/* add the amount indicated by res to the cred current use */
#define credres_add(c,r,f) { \
	(c)->current.f += (r)->f; \
	assert((c)->current.f <= (c)->limits.f); \
}

/* subtract the amount indicated by res to the cred current use */
#define credres_sub(c,r,f) { \
	(c)->current.f -= (r)->f; \
	assert((c)->current.f >= 0); \
}

ani_error_t
ani_cred_resource_alloc(ani_cred_t *cred, resource_t *res)
{
	resource_t nres;

	assert_ani_cred_valid(cred);

	nres = *res;
	if (res->threads == RESOURCE_ALL)
		nres.threads = cred->limits.threads - cred->current.threads;
	else if (credres_check(cred, res, threads))
		return ani_error_set(EPERM,
				     "Failed thread limit permission check");

	if (res->ppages == RESOURCE_ALL)
		nres.ppages = cred->limits.ppages - cred->current.ppages;
	else if (credres_check(cred, res, ppages))
		return ani_error_set(EPERM,
				     "Failed physical page limit permission check");

	*res = nres;
	credres_add(cred, res, threads);
	credres_add(cred, res, ppages);

	assert_ani_cred_valid(cred);

	return ani_error_clear();
}

void
ani_cred_resource_free(ani_cred_t *cred, resource_t *res)
{
	assert_ani_cred_valid(cred);
	credres_sub(cred, res, threads);
	credres_sub(cred, res, ppages);
	assert_ani_cred_valid(cred);
}

