/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _DEMUX_IMPL_H_
#define _DEMUX_IMPL_H_

/*
 * Demux ID
 */
typedef void *demux_id_t;

/*
 * Demux "match" function.
 *
 * Determines if the given packet (pdata/plen) meets the critera of the
 * indicated demux data (ddata).  Returns the offset of the first byte
 * in the packet AFTER the part that matches.  Returns -1 if there is no
 * match.
 */
typedef int (*demux_func_t)(void *ddata, void *pdata, int plen);

/*
 * A demultiplexor (aka, filter or classifier).
 *
 * Describes a function and a block of demux data.  The function is called
 * on every packet.  Demuxs are chained together in lists ordered by their
 * priority value.  Demux_apply takes such a list of demuxs and invokes each
 * one in order til one matches, hence higher priority (more specific)
 * filters will match a packet in preference to lower priority ones.
 * A null demux function indicates a default "always match" action.
 */
struct demux {
	struct demux	*next;	/* next demux in list */
	int		prio;	/* the "priority" for sorting */
	demux_id_t	id;	/* caller supplied identifier for demux */
	demux_func_t	func;	/* demux function to invoke */
	void		*data;	/* demux function-specific data */
};
typedef struct demux demux_t;

void demux_init(void);
ani_error_t demux_create(demux_id_t id, demux_func_t func, void *data,
			 demux_t **out_demux);
void demux_destroy(demux_t *demux);
ani_error_t demux_add(demux_t **list, demux_t *demux, int prio);
demux_t *demux_find(demux_t *list, demux_id_t id);
void demux_remove(demux_t **list, demux_t *demux);
demux_id_t demux_apply(demux_t *list, void *data, int len);

#endif /* _DEMUX_IMPL_H_ */
