#
# Copyright (c) 1999, 2000 University of Utah and the Flux Group.
# All rights reserved.
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

TARGET =	libnodeos.a
TARGET_P =	libnodeos_p.a

SRCS =		main.c flow.c threadpool.c thread.c credential.c resource.c \
		mempool.c channel.c pbuffer.c synch.c demux.c event.c \
		demuxkey.c hnode.c if.c ani_time.c \
		chan_clock.c chan_eth.c chan_socket.c \
		an_mutex.c an_cond.c an_time.c an_info.c \
		an_cred.c an_flow.c an_thread.c an_mempool.c \
		an_chan.c an_demuxkey.c an_pbuf.c \
		an_gprof.c an_event.c an_debug.c \
		ani_moab_setup.c ani_error.c ani_debug.c \
		uninlined.c

vpath %.c $(SRCDIR) $(SRCDIR)/pq $(SRCDIR)/os

include $(NODEOS_SRCDIR)/arch.mk


ifeq ($(HOST_OS),oskit)
ARCH_SRCS = 	oskit.c oskit_dev.c oskit_mem.c oskit_net.c \
		oskit_gprof.c oskit_fs.c oskit_timer.c \
		oskit_hostname.c \
		pqueue.c pq.c
endif

ifeq ($(HOST_OS),oskitunix)
ARCH_SRCS =	oskitunix.c oskitunix_mem.c oskitunix_dev.c \
		oskitunix_gprof.c oskitunix_fs.c \
		oskit_timer.c oskit_hostname.c \
		pqueue.c pq.c
endif

ARCH_SRCS_UNIX_COMMON = unix_eth.c unix_gprof.c unix_fs.c \
		unix_timer.c unix_panic.c unix_hostname.c \
		unix_init.c

ifeq ($(HOST_OS),linux)
ARCH_SRCS =	linux_dumpstate.c linux_chaneth.c $(ARCH_SRCS_UNIX_COMMON)
endif

ifeq ($(HOST_OS),solaris)
ARCH_SRCS =	solaris_dumpstate.c solaris_chaneth.c $(ARCH_SRCS_UNIX_COMMON)
endif

ifeq ($(HOST_OS),freebsd)
ARCH_SRCS =	freebsd_dumpstate.c freebsd_chaneth.c $(ARCH_SRCS_UNIX_COMMON)
endif



FILES = 	$(SRCS) $(addprefix os/,$(ARCH_SRCS))
FQSRCS =	$(addprefix $(SRCDIR)/,$(sort $(FILES)))
DEPFILES:=	$(patsubst %.c,%.d,$(FILES))

OBJS := $(patsubst %.c,%.o,$(notdir $(FQSRCS)))
POBJS := $(patsubst %.c,%.po,$(notdir $(FQSRCS)))

#CFLAGS_WARNINGS=-W -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wnested-externs -Winline
CFLAGS_WARNINGS = -Wwrite-strings -Wstrict-prototypes -Winline

ifneq ($(HOST_OS),solaris)	# these are too annoying on solaris
CFLAGS_WARNINGS += -Wcast-align -Wshadow
endif

CPPFLAGS = $(OPT_CFLAGS) $(ARCH_CFLAGS) -I$(NODEOS_SRCDIR) -I$(NODEOS_SRCDIR)/src
CFLAGS += -Wall $(CFLAGS_WARNINGS) $(CPPFLAGS)

ifeq ($(ENABLE_PROFILING),yes)
all: all_ all_p
install: install_ install_p
else
all: all_
install: install_
endif

all_: $(TARGET)

all_p: $(TARGET) $(TARGET_P)

install_dirs:
	@test -d $(INSTALL_TOPDIR) || mkdir $(INSTALL_TOPDIR)
	@test -d $(INSTALL_LIBDIR) || mkdir $(INSTALL_LIBDIR)

install_: install_dirs $(TARGET)
	$(SILENT_MAKE_INFORM) "Installing $(TARGET)."
	$(INSTALL_DATA) $(TARGET) $(INSTALL_LIBDIR)

install_p: install_dirs $(TARGET_P)
	$(SILENT_MAKE_INFORM) "Installing $(TARGET_P)."
	$(INSTALL_DATA) $(TARGET_P) $(INSTALL_LIBDIR)


$(TARGET): $(OBJS)
	$(SILENT_MAKE_INFORM) "Creating library $@."
	$(AR) cru $@ $^
	$(RANLIB) $@

$(TARGET_P): $(POBJS)
	$(SILENT_MAKE_INFORM) "Creating library $@."
	$(AR) cru $@ $^
	$(RANLIB) $@

clean:
	$(SILENT_MAKE_INFORM) "Removing *.o *.po $(TARGET) $(TARGET_P)."
	rm -f .depend core *.core *.d *.o *.po $(TARGET) $(TARGET_P)

distclean: clean
	$(SILENT_MAKE_INFORM) "Scrubbing nodeos library directory."
	rm -f GNUmakefile

depend: 
	@echo "WARNING: depend target is deprectated."

mkdebug:
	@echo "ARCH_SRCS = $(ARCH_SRCS)"

-include $(DEPFILES)

.PHONY: depend distclean clean mkdebug
.PHONY: all all_ all_p
.PHONY: install install_dirs install_ install_p 

