/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _NODEOS_SRC_OS_H_
#define _NODEOS_SRC_OS_H_

#include <nodeos/an_error.h>
#include <nodeos/impl/ani_thread.h>

/*
 * The host-os-specific interfaces.  There are five (somewhat)
 * separate implementations of these functions for: 
 *   Solaris, Linux, FreeBSD, OSKit and OSKit-unix.
 * All the relevant files are in the os/ subdirectory.
 *
 * XXX most of these interfaces need to get an 'ani_' prefix.
 */


/*
 * Basic initialization interfaces.
 */
void os_init_basic(void);
void os_init_mem(void);
void os_init_devices(void);
void os_init_net(void);
void os_init_fs(void);


int os_sched_setprio(ani_thread_t *thread, int nprio);
void os_dump_state(void *state);

/*
 * Gprof interfaces
 */
void os_gprof_init(void);
void os_gprof_start(void);
void os_gprof_stop(void);

/*
 * Info interfaces
 */
const char *ani_os_gethostname(void);
int ani_os_sethostname(const char *name);

/*
 * Timer interfaces
 */
void *os_timer_init(void (*func)(void *, void *), void *farg,
		    unsigned long *timerres);
void os_timer_deinit(void *handle);
int os_timer_start(void *handle, unsigned long timerres);

void panic(const char *fmt, ...);

#endif /* _NODEOS_SRC_OS_H_ */
