/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <stdio.h>
#include <signal.h>

#include "os.h"

/*
 * Linux signal-state dump
 */

static void
sigcontext_dump_stack_trace(struct sigcontext *scp)
{
	unsigned int *fp, i;

	fp = (unsigned int *)scp->ebp;

	printf("Backtrace:");
	for (i = 0; i < 16; i++) {
		if ((i % 8) == 0)
			printf("\n");

		fp = (int *)(*fp);
		if (!(*(fp + 1) && *fp))
			break;
		printf(" %08x", *(fp + 1));
	}
	printf("\n");
}

void
os_dump_state(void *state)
{
	struct sigcontext *scp = state;
	static int indump;

	if (indump++)
		return;

	printf("EAX %08lx EBX %08lx ECX %08lx EDX %08lx\n",
	       scp->eax, scp->ebx, scp->ecx, scp->edx);
	printf("ESI %08lx EDI %08lx EBP %08lx ESP %08lx\n",
	       scp->esi, scp->edi, scp->ebp, scp->esp);
	printf("EIP %08lx EFLAGS %08lx\n", scp->eip, scp->eflags);
	printf("CS %04x SS %04x DS %04x ES %04x\n",
	       scp->cs & 0xffff, scp->ss & 0xffff,
	       scp->ds & 0xffff, scp->es & 0xffff);
	sigcontext_dump_stack_trace(scp);
	indump = 0;
}

