/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Hodge-podge of stuff for OSKit
 */
#include <oskit/clientos.h>
#include <oskit/startup.h>
#include <oskit/threads/pthread.h>

#include "os.h"
#include "options.h"

#include <nodeos/an_thread.h>
#include <nodeos/impl/an_error.h>
#include <nodeos/impl/ani_thread.h>

/*
 * Get the basic OSKit ``stuff'' up and running.
 */
void
os_init_basic(void)
{
	oskit_clientos_init_pthreads();
	start_clock();
	start_pthreads();
}

/*
 * Set the priority of the given thread.
 * This routine should map standard NodeOS priority values into
 * implementation specific priorities.
 */
int
os_sched_setprio(ani_thread_t *thread, int nprio)
{
	int prio, err;

	switch (nprio) {
	case AN_THREAD_PRIO_IDLE:
		prio = PRIORITY_MIN;
		break;
	case AN_THREAD_PRIO_EXCLUSIVE:
		prio = PRIORITY_MAX;
		break;
	case AN_THREAD_PRIO_NORMAL:
		prio = PRIORITY_NORMAL;
		break;
	default:
		return EINVAL;
	}
	err = oskit_pthread_setprio(thread->id, prio);
	if (err == 0)
		thread->pprio.priority = prio;

	return err;
}

void
os_dump_state(void *state)
{
	sigcontext_dump((struct sigcontext *)state);
}
