/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Hodge-podge of stuff for OSKit
 */
#include <stdio.h>

#include <oskit/dev/dev.h>
#include <oskit/dev/osenv.h>
#include <oskit/dev/blk.h>
#include <oskit/dev/ethernet.h>
#include <oskit/dev/bus.h>
#include <oskit/dev/linux.h>

#include <oskit/clientos.h>
#include <oskit/startup.h>
#include <oskit/threads/pthread.h>

#include "options.h"

#ifdef HAVE_DEVICES
static int verbose = 1;
#endif

/*
 * Initialize and probe device drivers.
 */
void
os_init_devices(void)
{
#ifdef HAVE_DEVICES
	osenv_process_lock();
	start_devices();
	osenv_process_unlock();
#endif
}

#ifdef HAVE_DEVICES
/*
 * This is our own version of start devices so we can be more selective.
 * Comment this out, if you want all the enet and disk devices.
 */
void
start_devices(void)
{
	static void init_net(void);
	static void init_disk(void);
	oskit_osenv_t *osenv;
	static int run;
	
	if (run++)
		return;

	printf("Initializing device drivers...\n");
	osenv = start_osenv();
	oskit_dev_init(osenv);
	oskit_linux_init_osenv(osenv);

#ifdef HAVE_NET
	init_net();
#endif
#ifdef HAVE_DISK
	init_disk();
#endif

	if (verbose)
		oskit_dump_drivers();

	printf("Probing devices...\n");
	oskit_dev_probe();

	if (verbose)
		oskit_dump_devices();
}

#ifdef HAVE_NET
/*
 * Equiv of oskit_linux_init_ethernet
 *
 * Insert individual drivers you want or call oskit_linux_init_ethernet
 * for all of them.
 */
static void
init_net(void)
{
#ifdef i386
	oskit_linux_init_ethernet_tulip();
	oskit_linux_init_ethernet_eepro100();
#endif
#ifdef arm32
	oskit_linux_init_ethernet_cs89x0();
#endif
}

/*
 * XXX override startup library.
 *
 * This routine would be pulled in by startup_network_router and the
 * default version would pull in all enet drivers.
 */
void
start_net_devices(void)
{
}

#endif

#ifdef HAVE_DISK
/*
 * Equiv of oskit_linux_init_blk
 *
 * Insert individual drivers you want or call oskit_linux_init_blk
 * for all of them.
 */
static void
init_disk(void)
{
	oskit_linux_init_ide();
	oskit_linux_init_scsi_ncr53c8xx();
	oskit_linux_init_scsi_aic7xxx();
}
#else

/*
 * XXX guess what?  A hack to keep from dragging in a ton o' crap!
 * The default Linux device_setup pulls in all the block device code:
 * ide, scsi, you name it.  Turning on interrupts seems to be the only
 * thing we need if we don't care about disk devices.
 */
void
FDEV_LINUX_device_setup(void)
{
	osenv_intr_enable();
}
#endif
#endif

