/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _PQ_IMPL_H_
#define _PQ_IMPL_H_

#define STATS

#ifdef	OSKIT
#include <oskit/dev/dev.h>
#include <oskit/io/netio.h>
#include <oskit/io/bufio.h>
#else
#include "unix_eth.h"
#endif

#include "pq.h"
#include "pqueue.h"
#include "queue.h"
#include <nodeos/impl/ani_ilock.h>

/*
 * Stride Packet Scheduler.
 */
struct pq_impl {
	struct pq_impl		*schedlink;	/* Link */
	int			refcount;	/* Sanity check */

	/*
	 * Need to protect the scheduler from concurrent access.
	 */
	ani_ilock_t	 lock;

	/*
	 * A "priority" queue of child nodes that *currently* have
	 * something to send. The child with the smallest pass value is the
	 * one to be chosen next. Currently, its a simple (read: slow)
	 * priority queue. Children are on this list *only* if they having
	 * something to send!
	 */
	queue_head_t		clients;
	
	/*
	 * These cover the children of this node. Known as "global_"
	 * in a flat model. If this node is a leaf node, then they
	 * hold values for just the leaf, obviously.
	 */
	int			tickets;	
	int			stride;	
	long long		pass;
	long long		remain;
	long long		bytes_transferred;

	/*
	 * A packet queue for the leaf nodes. Might be null, with a callout
	 * defined to grab the next packet. 
	 */
	pqueue_t	       *pqueue;
	char			ready;		/* Got packets! */

#ifdef  OSKIT
	/*
	 * Device is represented by an oskit_netio_t.
	 */
	oskit_netio_t	       *netio;
#else
	/*
	 * Device is represented by a file descriptor.
	 */
	int			devfd;
#endif
	int			burstlen;

#ifdef  STATS
	int			sent;
	int			stalls;
	int			drops;
#endif
};
typedef struct pq_impl pq_impl_t;

/*
 * Size of the physical queues used by the leaf nodes. Might be zero
 * if the queuing exists outside the system.
 */
#define DEFAULT_PACKET_QUEUE_LENGTH	200

/*
 * Default packet burst length.
 */
#define DEFAULT_BURST_LENGTH		10

/*
 * The Quantum. The cost charged for a packet is a function of the
 * number of bytes in the packet / by the quantum. Needs to be a
 * largish constant representing the cost of a complete quantum, which
 * in this case is the number of bytes that some maximum sized packet
 * would be.
 */
#define QUANTUM		8192

/*
 * The Stride constant. A large number so that stride can be represented
 * as a high precision fixed point integer instead of a float (stride
 * is the reciprocal of tickets).
 *
 * WARNING: This number is somewhat dependent on the value of MAXTICKETS
 * in pq.h. Probably want to look at that value before changing this one.
 */
#define STRIDE		(1 << 20)

/*
 * Keep ifdefs to a minimum.
 */
#ifdef  STATS
#define DOSTAT(x)	x
#else
#define DOSTAT(x)
#endif

#endif /* _PQ_IMPL_H_ */
