/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _PQUEUE_H_
#define _PQUEUE_H_

#include <assert.h>

/*
 * Simple fixed sized packet queue.
 */
struct pqueue {
	int		qhead;
	int		qtail;
	int		qmax;
	int		qdrops;
	struct pks {
		void	*p;
		int	size;	/* Need size to compute charge */
	} packets[0];
};
typedef struct pqueue pqueue_t;

#define pqueue_nexti(i)	((i != pqueue->qmax) ? i+1 : 0)

/*
 * Enqueue a packet!
 */
static inline int
pqueue_enqueue(pqueue_t *pqueue, void *p, int size)
{
	int next = pqueue_nexti(pqueue->qtail);
	
	if (next != pqueue->qhead) {
		pqueue->packets[pqueue->qtail].p    = p;
		pqueue->packets[pqueue->qtail].size = size;
		pqueue->qtail = next;
		return 0;
	} else {
		pqueue->qdrops++;
		return -1;
	}
}

static inline int
pqueue_dequeue(pqueue_t *pqueue, void **p, int *size)
{
	if (pqueue->qhead != pqueue->qtail) {
		*p    = pqueue->packets[pqueue->qhead].p;
		*size = pqueue->packets[pqueue->qhead].size;
		pqueue->qhead = pqueue_nexti(pqueue->qhead);
		return 0;
	}
	else
		return -1;
}

static inline void
pqueue_first(pqueue_t *pqueue, void **p, int *size)
{
	*p    = pqueue->packets[pqueue->qhead].p;
	*size = pqueue->packets[pqueue->qhead].size;
}

static inline void
pqueue_advance(pqueue_t *pqueue)
{
	pqueue->qhead = pqueue_nexti(pqueue->qhead);
}

static inline int
pqueue_empty(pqueue_t *pqueue)
{
	return pqueue->qhead == pqueue->qtail;
}

static inline int
pqueue_full(pqueue_t *pqueue)
{
	int next = pqueue_nexti(pqueue->qtail);
	
	return next == pqueue->qhead;
}

pqueue_t       *pqueue_alloc(int maxsize);
void		pqueue_free(pqueue_t *pqueue);
void		pqueue_dump(pqueue_t *pqueue);

#endif /* _PQUEUE_H_ */
